function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import { State, LevelState, TermQuery, TermsBucket, FilterBucket, SignificantTermsBucket, BoolShould, BoolMust } from "searchkit";

import { AccessorBase } from "./AccessorBase";

import { map } from "lodash";
import { each } from "lodash";
import { compact } from "lodash";
import { take } from "lodash";
import { omitBy } from "lodash";
import { isUndefined } from "lodash";

// export interface HierarchicalFacetAccessorOptions {
//     fields: Array<string>
//     size: number
//     id: string
//     title: string
//     orderKey?: string
//     orderDirection?: string
// }

export var SignTermsAccessor = function (_AccessorBase) {
    _inherits(SignTermsAccessor, _AccessorBase);

    function SignTermsAccessor(key, options /*HierarchicalFacetAccessorOptions*/) {
        _classCallCheck(this, SignTermsAccessor);

        var _this = _possibleConstructorReturn(this, _AccessorBase.call(this, key));

        _this.state = new LevelState();

        _this.options = options;
        _this.computeUuids();
        return _this;
    }

    SignTermsAccessor.prototype.computeUuids = function computeUuids() {
        var _this2 = this;

        this.uuids = map(this.options.fields, function (field) {
            return _this2.uuid + field;
        });
    };

    SignTermsAccessor.prototype.onResetFilters = function onResetFilters() {
        this.resetState();
    };

    SignTermsAccessor.prototype.getBuckets = function getBuckets(level) {
        var field = this.options.fields[level];
        // var aggs = this.getAggregations([this.options.id, "buckets"], [])
        var aggs = this.getAggregations([this.options.id, field, field, "buckets"], []);
        return aggs;
    };

    SignTermsAccessor.prototype.getOrder = function getOrder() {
        if (this.options.orderKey) {
            var _ref;

            var orderDirection = this.options.orderDirection || "asc";
            return _ref = {}, _ref[this.options.orderKey] = orderDirection, _ref;
        }
    };

    SignTermsAccessor.prototype.buildSharedQuery = function buildSharedQuery(query) {
        var _this3 = this;

        each(this.options.fields, function (field, i) {
            var filters = _this3.state.getLevel(i);
            var parentFilter = _this3.state.getLevel(i - 1);
            var isLeaf = !_this3.state.levelHasFilters(i + 1);
            var filterTerms = map(filters, TermQuery.bind(null, field));

            if (filterTerms.length > 0) {
                query = query.addFilter(_this3.uuids[i], filterTerms.length > 1 ? BoolShould(filterTerms) : filterTerms[0]);
            }

            if (isLeaf) {
                var selectedFilters = map(filters, function (filter) {
                    return {
                        id: _this3.options.id,
                        name: _this3.translate(parentFilter[0]) || _this3.options.title || _this3.translate(field),
                        value: _this3.translate(filter),
                        remove: function remove() {
                            _this3.state = _this3.state.remove(i, filter);
                        }
                    };
                });
                query = query.addSelectedFilters(selectedFilters);
            }
        });

        console.log("Build Shared Query");
        query.printJSON();
        return query;
    };

    SignTermsAccessor.prototype.buildOwnQuery = function buildOwnQuery(query) {
        var _this4 = this;

        var filters = this.state.getValue();
        var field = this.options.fields[0];

        var lvlAggs = compact(map(this.options.fields, function (field, i) {
            if (_this4.state.levelHasFilters(i - 1) || i == 0) {
                if (i == 0) {
                    var bucketType = TermsBucket;
                } else if (i == 1) {
                    var bucketType = SignificantTermsBucket;
                }
                return FilterBucket(field, query.getFiltersWithKeys(take(_this4.uuids, i)), bucketType(field, field, omitBy({
                    size: _this4.options.size, order: _this4.getOrder()
                }, isUndefined)));
            }
        }));

        // var aggs = FilterBucket(
        //     this.options.id,
        //     query.getFiltersWithoutKeys(this.uuids),
        //     ...lvlAggs
        // )

        // export function TermsBucket(key, field, options: TermsBucketOptions = {}, ...childAggs) {
        //     return AggsContainer(key, {
        //         terms: assign({ field }, options)
        //     }, childAggs)
        // }

        // export function SignificantTermsBucket(key, field, options={}, ...childAggs){
        //     return AggsContainer(key, {significant_terms:assign({field}, options)}, childAggs)
        // }

        var aggs = FilterBucket.apply(undefined, [this.options.id, query.getFiltersWithoutKeys(this.uuids)].concat(lvlAggs));

        // var aggs = TermsBucket(
        //     this.options.id,
        //     "meta.subject",
        //     {},
        //     SignificantTermsBucket(
        //         this.options.id,
        //         "meta.text"
        //     )
        // )

        console.log("Build Own Query");
        query.printJSON();
        var queryAfterAggs = query.setAggs(aggs);
        console.log("Build Aggs Query");
        queryAfterAggs.printJSON();
        return queryAfterAggs;
    };

    return SignTermsAccessor;
}(AccessorBase);