"use strict";

exports.__esModule = true;
exports.SignFacetAccessor = undefined;

var _class, _temp;

var _searchkit = require("searchkit");

var _lodash = require("lodash");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SignFacetAccessor = exports.SignFacetAccessor = (_temp = _class = function (_FilterBasedAccessor) {
  _inherits(SignFacetAccessor, _FilterBasedAccessor);

  function SignFacetAccessor(key, options) {
    _classCallCheck(this, SignFacetAccessor);

    var _this = _possibleConstructorReturn(this, _FilterBasedAccessor.call(this, key, options.id));

    _this.state = new _searchkit.ArrayState();
    _this.translations = SignFacetAccessor.translations;

    _this.options = options;
    _this.defaultSize = options.size;
    _this.options.facetsPerPage = _this.options.facetsPerPage || 50;
    _this.size = _this.defaultSize;
    _this.loadAggregations = (0, _lodash.isUndefined)(_this.options.loadAggregations) ? true : _this.options.loadAggregations;
    if (options.translations) {
      _this.translations = (0, _lodash.assign)({}, _this.translations, options.translations);
    }
    _this.options.fieldOptions = _this.options.fieldOptions || { type: "embedded" };
    _this.options.fieldOptions.field = _this.key;
    _this.fieldContext = (0, _searchkit.FieldContextFactory)(_this.options.fieldOptions);
    return _this;
  }

  SignFacetAccessor.prototype.getRawBuckets = function getRawBuckets() {
    return this.getAggregations([this.uuid, this.fieldContext.getAggregationPath(), this.key, "buckets"], []);
  };

  SignFacetAccessor.prototype.getBuckets = function getBuckets() {
    var rawBuckets = this.getRawBuckets();
    var keyIndex = (0, _lodash.keyBy)(rawBuckets, "key");
    var inIndex = function inIndex(filter) {
      return !!keyIndex[filter];
    };
    var missingFilters = [];
    (0, _lodash.each)(this.state.getValue(), function (filterKey) {
      if (keyIndex[filterKey]) {
        var filter = keyIndex[filterKey];
        filter.selected = true;
      } else {
        missingFilters.push({
          key: filterKey, missing: true, selected: true
        });
      }
    });
    var buckets = missingFilters.length > 0 ? missingFilters.concat(rawBuckets) : rawBuckets;

    return buckets;
  };

  SignFacetAccessor.prototype.getDocCount = function getDocCount() {
    return this.getAggregations([this.uuid, this.fieldContext.getAggregationPath(), "doc_count"], 0);
  };

  SignFacetAccessor.prototype.getCount = function getCount() {
    return this.getAggregations([this.uuid, this.fieldContext.getAggregationPath(), this.key + "_count", "value"], 0);
  };

  SignFacetAccessor.prototype.setViewMoreOption = function setViewMoreOption(option) {
    this.size = option.size;
  };

  SignFacetAccessor.prototype.getMoreSizeOption = function getMoreSizeOption() {
    var option = { size: 0, label: "" };
    var total = this.getCount();
    var facetsPerPage = this.options.facetsPerPage;
    if (total <= this.defaultSize) return null;

    if (total <= this.size) {
      option = { size: this.defaultSize, label: this.translate("facets.view_less") };
    } else if (this.size + facetsPerPage >= total) {
      option = { size: total, label: this.translate("facets.view_all") };
    } else if (this.size + facetsPerPage < total) {
      option = { size: this.size + facetsPerPage, label: this.translate("facets.view_more") };
    } else if (total) {
      option = null;
    }

    return option;
  };

  SignFacetAccessor.prototype.isOrOperator = function isOrOperator() {
    return this.options.operator === "OR";
  };

  SignFacetAccessor.prototype.getBoolBuilder = function getBoolBuilder() {
    return this.isOrOperator() ? _searchkit.BoolShould : _searchkit.BoolMust;
  };

  SignFacetAccessor.prototype.getOrder = function getOrder() {
    if (this.options.orderKey) {
      var _ref;

      var orderDirection = this.options.orderDirection || "asc";
      return _ref = {}, _ref[this.options.orderKey] = orderDirection, _ref;
    }
  };

  SignFacetAccessor.prototype.buildSharedQuery = function buildSharedQuery(query) {
    var _this2 = this;

    var filters = this.state.getValue();
    var filterTerms = (0, _lodash.map)(filters, function (filter) {
      return _this2.fieldContext.wrapFilter((0, _searchkit.TermQuery)(_this2.key, filter));
    });
    var selectedFilters = (0, _lodash.map)(filters, function (filter) {
      return {
        name: _this2.options.title || _this2.translate(_this2.key),
        value: _this2.translate(filter),
        id: _this2.options.id,
        remove: function remove() {
          return _this2.state = _this2.state.remove(filter);
        }
      };
    });
    var boolBuilder = this.getBoolBuilder();
    if (filterTerms.length > 0) {
      query = query.addFilter(this.uuid, boolBuilder(filterTerms)).addSelectedFilters(selectedFilters);
    }

    return query;
  };

  SignFacetAccessor.prototype.buildOwnQuery = function buildOwnQuery(query) {
    if (!this.loadAggregations) {
      return query;
    } else {
      var filters = this.state.getValue();
      var excludedKey = this.isOrOperator() ? this.uuid : undefined;
      return query.setAggs(_searchkit.FilterBucket.apply(undefined, [this.uuid, query.getFiltersWithoutKeys(excludedKey)].concat(this.fieldContext.wrapAggregations((0, _searchkit.SignificantTermsBucket)(this.key, this.key, (0, _lodash.omitBy)({
        size: this.size,
        order: this.getOrder(),
        include: this.options.include,
        exclude: this.options.exclude,
        min_doc_count: this.options.min_doc_count
      }, _lodash.isUndefined)), (0, _searchkit.CardinalityMetric)(this.key + "_count", this.key)))));
    }
  };

  return SignFacetAccessor;
}(_searchkit.FilterBasedAccessor), _class.translations = {
  "facets.view_more": "View more",
  "facets.view_less": "View less",
  "facets.view_all": "View all"
}, _temp);